-- Skada Scroll by oscarucb
local Skada = Skada
local bars = Skada:GetModule("BarDisplay")
local mod = Skada:NewModule("Scroll")
local addonName = "SkadaScroll"
local texpath = "Interface\\AddOns\\"..addonName.."\\scroll"

mod.scrollspeed = 2.0
mod.elap = 0
mod.debugging = false
local function debug(msg)
  if mod.debugging then
    print(addonName..": "..msg)
  end
end

function mod.ShowCursor(win)
  --SetCursor(texpath)
  SetCursor("")
  mod.ScrollIcon = mod.ScrollIcon or {}
  if not mod.ScrollIcon[win] then
    local f = CreateFrame("Frame", nil, win.bargroup)
    f:SetFrameStrata("TOOLTIP")
    f:SetSize(32,32)
    f:SetPoint("CENTER")
    local t = f:CreateTexture(nil, "OVERLAY")
    t:SetTexture(texpath)
    t:SetAllPoints(f)
    t:Show()
    mod.ScrollIcon[win] = f
  end
  mod.ScrollIcon[win]:Show()
end

function mod.HideCursor(win)
  mod.ScrollIcon[win]:Hide()
  ResetCursor()
end

function mod.BeginScroll(win)
  debug("BeginScroll")
  mod.ypos = select(2,GetCursorPosition())
  mod.scrolling = win
  mod.ShowCursor(win)
end

function mod.EndScroll(win)
  debug("EndScroll")
  mod.scrolling = nil
  mod.HideCursor(win)
end

function mod.OnUpdate(f, elap)
  local win = mod.scrolling
  if not win then return end
  if not IsMouseButtonDown("MiddleButton") then
    mod.EndScroll(win)
    return
  end
  mod.ShowCursor(win)
  mod.elap = mod.elap + elap
  if mod.elap < 0.1 then return end
  mod.elap = 0
  local newpos = select(2,GetCursorPosition())
  local step = (win.db.barheight + win.db.barspacing) / (win.bargroup:GetEffectiveScale() * mod.scrollspeed)
  while math.abs(newpos - mod.ypos) > step do
    if newpos > mod.ypos then
       bars:OnMouseWheel(win, nil, 1)
       mod.ypos = mod.ypos + step
    else
       bars:OnMouseWheel(win, nil, -1)
       mod.ypos = mod.ypos - step
    end
  end
end

mod.frame = CreateFrame("Button", "SkadaScrollHiddenFrame", UIParent)
mod.frame:SetScript("OnUpdate", mod.OnUpdate)

local hooked = {}
function mod.Create(self,win) 
  debug("Create")
  win.bargroup:HookScript("OnMouseDown", function(frame, button) if button == "MiddleButton" then mod.BeginScroll(win) end end)
  hooksecurefunc(win.bargroup,"SortBars", function() mod.HookMore(win) end)
end
hooksecurefunc(bars,"Create", mod.Create)

local function BarClick(bar, button)
  if button == "MiddleButton" then 
     mod.BeginScroll(bar.scrollwin)
  elseif hooked[bar] then
     (hooked[bar])(bar, button)
  end
end

function mod.HookMore(win)
  if not hooked[win] then
    debug("HookMore")
    local bars = win.bargroup:GetBars()
    if bars then
      for name, bar in pairs(bars) do
         local old = bar:GetScript("OnMouseDown")
         if old ~= BarClick then
            bar:SetScript("OnMouseDown", BarClick)
            bar.scrollwin = win
            hooked[bar] = old
         end
      end
    end
    hooked[win] = true
  end
end

function mod.CreateBar(self, win, name, label, value, maxvalue, icon, o)
   debug("CreateBar: "..name)
   hooked[win] = false
end
hooksecurefunc(bars,"CreateBar", mod.CreateBar)


